#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <etherpix/etherpix.h>

#ifdef WINDOWS
#include <winsock.h>
#endif

/* just an example for some data */
typedef struct userdata_s {
  int i;
  float f;
  char c;
} userdata_t;

/* callback function called during display creation,
   the p_ctx parameter is the userdata supplied in the call
   to etp_display_create() */
void msg(void *p_ctx, etp_msg_type_t type, const char *fmt, ...)
{
  userdata_t *p_userdata = (userdata_t *)p_ctx;
  va_list va;

  printf("msg (%d %f %c): ",
         p_userdata->i, p_userdata->f,
         p_userdata->c); /* make use of the userdata */

  /* type tells if the message is an error, a warning or an information */
  switch (type) {
    case etp_msg_type_err:
      printf("ERROR: ");
      break;
    case etp_msg_type_warn:
      printf("WARNING: ");
      break;
    case etp_msg_type_info:
      printf("INFORMATION: ");
      break;
  }

  /* fmt is a printf-style format string,
     the arguments are passed exactly as for printf */
  va_start(va, fmt);
  vprintf(fmt, va);
  va_end(va);
}

int main(int argc, char *argv[])
{
  const char *config;
  userdata_t userdata = { 42, 42.0, '!' }; /* just an example */
  etp_display_t *p_display;
  unsigned int width, height, i, x, y;
  etp_u8_t *p_image;

  if (argc < 2) {
    printf("usage: %s <config.etp>\n", argv[0]);
    return -1;
  }
  config = argv[1];

#ifdef WINDOWS
  {
    WSADATA data;
    WSAStartup(MAKEWORD(1,1), &data);
  }
#endif

  /* create a display, take the configuration from a file,
     deliver messages to the callback function msg(),
     the last parameter is passed as first parameter to msg()
     and can be used to supply some user defined data */
  p_display = etp_display_create(config, msg, (void *)&userdata);
  if (!p_display) {
    printf("could not create display\n");
    return -1;
  }

  etp_display_get_size(p_display, &width, &height);
  printf("width:  %5u\nheight: %5u\n", width, height);

  /* create an image with a nice fade:
     black to red from left to right,
     black to green from top to bottom */
  p_image = (etp_u8_t *)malloc(height * width * 3);
  if (!p_image) {
    etp_display_free(p_display);
    printf("out of memory\n");
    return -1;
  }
  i = 0;
  for (y = 0; y < height; ++y) {
    for (x = 0; x < width; ++x) {
      p_image[i++] = (etp_u8_t)(255.0*x/(width-1)+0.5); /* red */
      p_image[i++] = (etp_u8_t)(255.0*y/(height-1)+0.5); /* green */
      p_image[i++] = 0; /* no blue */
    }
  }

  /* output image */
  etp_display_data_fmt(p_display, p_image,
                       3, /* consecutive pixels are 3 bytes apart */
                       width*3, /* consecutive lines are width*3 bytes apart */
                       0, 0, width, height, etp_pixfmt_rgb24);
  etp_display_send(p_display);

  free(p_image);
  etp_display_free(p_display);

#ifdef WINDOWS
  WSACleanup();
#endif

  return 0;
}

